import os

JOBHUB_HOST = os.getenv("FL_JOBHUB_HOST", "101.201.215.196")
JOBHUB_PORT = int(os.getenv("FL_JOBHUB_PORT", "7901"))
SHARED_SECRET = os.getenv("FL_JOBHUB_SECRET", "01234567890123456789012345678901").encode()

JOBSET_ID = os.getenv("FL_JOBSET_ID", "fl_mnist")
SELF_JOB_ID = os.getenv("FL_JOB_ID", "1")

SCHEDULE_URL = "http://101.201.215.196:7891"
