
class BaseAdapter:

    def before_task(self, inputs, context: dict): ...

    def after_task(self, outputs, context: dict): ...

    def input_prepare(self, data_type: str, file_path: str): ...

    def output_prepare(self, data_type: str, file_path: str): ...

    def runtime_data_copy(self, remote_path: str, local_path: str): ...
