LEADER_PREFIX = "leader_"

INITIAL_MODEL_FROM_CLIENT = "initial_model_from_client"

DATA_OWNERSHIP_VERSION = "data_ownership_version"

FL_CONFIG = "fl_config"

DATA_OWNER = "data_owner_"

TOKEN = "token"
