from enum import Enum

PLATFORM = "platform"
PLATFORM_OPENI = "1865927992266461184"
PLATFORM_MODELARTS = "1790300942428540928"


class DataType:
    DATASET = "dataset"
    MODEL = "model"
    CODE = "code"


class NodeType:
    DATA_RETURN = "DataReturn"
    AI_TRAIN = "AITrain"
    BINDING = "Binding"
    START = "Start"
    END = "End"


class NodeID:
    START_ID = "-1"
    END_ID = "-2"
    FIRST_ID = "1"


class Platform:
    OPENI = "1865927992266461184"
    MODELARTS = "1790300942428540928"
    OCTOPUS = "octopus"


class SourceType(Enum):
    JCS = "JCS"
    LOCAL = "LOCAL"


class JCSAPI:
    PRESIGNED = "/presigned/object/download"


OUTPUT = "output"
DATASET_INPUT = "dataset_input"
CODE_INPUT = "code_input"
MODEL_INPUT = "model_input"
CURRENT_PLATFORM = "CURRENT_PLATFORM"
GATEWAY_ADDR = "GATEWAY_ADDR"
USER_ID = "USER_ID"
JCS_ADDR = "JCS_ADDR"
JCS_AK = "JCS_AK"
JCS_SK = "JCS_SK"
JOBHUB_ADDR = "JOBHUB_ADDR"
JOBHUB_PORT = "JOBHUB_PORT"
JOBHUB_JOBSET_ID = "JOBHUB_JOBSET_ID"
JOBHUB_JOB_ID = "JOBHUB_JOB_ID"
JOBHUB_SECRET = "JOBHUB_SECRET"
